@@ -6,6 +6,7 @@ require 'hamlit/html'
 require 'hamlit/escapable'
 require 'hamlit/force_escapable'
 require 'hamlit/dynamic_merger'
+require 'hamlit/ambles'
 
 module Hamlit
   class Engine < Temple::Engine
@@ -30,6 +31,7 @@ module Hamlit
     use Escapable
     use ForceEscapable
     filter :ControlFlow
+    use Ambles
     filter :MultiFlattener
     filter :StaticMerger
     use DynamicMerger
