@@ -6,7 +6,7 @@ module Hamlit
         text = node.value[:text].rstrip.gsub(/^/, prefix)
         if ::Hamlit::HamlUtil.contains_interpolation?(node.value[:text])
           # original: Haml::Filters#compile
-          text = ::Hamlit::HamlUtil.slow_unescape_interpolation(text).gsub(/(\\+)n/) do |s|
+          text = ::Hamlit::HamlUtil.unescape_interpolation(text).gsub(/(\\+)n/) do |s|
             escapes = $1.size
             next s if escapes % 2 == 0
             "#{'\\' * (escapes - 1)}\n"
