@@ -1,11 +1,10 @@
-require 'hamlit/parser/haml_error'
-require 'hamlit/parser/haml_options'
-require 'hamlit/parser/haml_compiler'
-require 'hamlit/parser/haml_parser'
+# frozen_string_literal: true
+
+require 'erb'
 
 module Hamlit
   # This module contains various helpful methods to make it easier to do various tasks.
-  # {Haml::Helpers} is automatically included in the context
+  # {Hamlit::HamlHelpers} is automatically included in the context
   # that a Haml template is parsed in, so all these methods are at your
   # disposal from within the template.
   module HamlHelpers
@@ -23,10 +22,10 @@ MESSAGE
 
       # Raises an error.
       #
-      # @raise [Haml::Error] The error
+      # @raise [Hamlit::HamlError] The error
       def to_s
-        raise ::Hamlit::HamlError.new(@message)
-      rescue ::Hamlit::HamlError => e
+        raise Hamlit::HamlError.new(@message)
+      rescue Hamlit::HamlError => e
         e.backtrace.shift
 
         # If the ErrorReturn is used directly in the template,
@@ -45,7 +44,7 @@ MESSAGE
 
       # @return [String] A human-readable string representation
       def inspect
-        "::Hamlit::HamlHelpers::ErrorReturn(#{@message.inspect})"
+        "Hamlit::HamlHelpers::ErrorReturn(#{@message.inspect})"
       end
     end
 
@@ -69,13 +68,13 @@ MESSAGE
     #
     #     context = Object.new
     #     class << context
-    #       include Haml::Helpers
+    #       include Hamlit::HamlHelpers
     #     end
     #     context.init_haml_helpers
     #     context.haml_tag :p, "Stuff"
     #
     def init_haml_helpers
-      @haml_buffer = ::Hamlit::HamlBuffer.new(haml_buffer, ::Hamlit::HamlOptions.new.for_buffer)
+      @haml_buffer = Hamlit::HamlBuffer.new(haml_buffer, HamlOptions.new.for_buffer)
       nil
     end
 
@@ -111,10 +110,7 @@ MESSAGE
     #   @yield The block within which to escape newlines
     def find_and_preserve(input = nil, tags = haml_buffer.options[:preserve], &block)
       return find_and_preserve(capture_haml(&block), input || tags) if block
-      tags = tags.each_with_object('') do |t, s|
-        s << '|' unless s.empty?
-        s << Regexp.escape(t)
-      end
+      tags = tags.map { |tag| Regexp.escape(tag) }.join('|')
       re = /<(#{tags})([^>]*)>(.*?)(<\/\1>)/im
       input.to_s.gsub(re) do |s|
         s =~ re # Can't rely on $1, etc. existing since Rails' SafeBuffer#gsub is incompatible
@@ -202,20 +198,19 @@ MESSAGE
     # @yield [item] A block which contains Haml code that goes within list items
     # @yieldparam item An element of `enum`
     def list_of(enum, opts={}, &block)
-      opts_attributes = opts.each_with_object('') {|(k, v), s| s << " #{k}='#{v}'"}
-      enum.each_with_object('') do |i, ret|
+      opts_attributes = opts.map { |k, v| " #{k}='#{v}'" }.join
+      enum.map do |i|
         result = capture_haml(i, &block)
 
         if result.count("\n") > 1
           result.gsub!("\n", "\n  ")
-          result = "\n  #{result.strip!}\n"
+          result = "\n  #{result.strip}\n"
         else
           result.strip!
         end
 
-        ret << "\n" unless ret.empty?
-        ret << %Q!<li#{opts_attributes}>#{result}</li>!
-      end
+        %Q!<li#{opts_attributes}>#{result}</li>!
+      end.join("\n")
     end
 
     # Returns a hash containing default assignments for the `xmlns`, `lang`, and `xml:lang`
@@ -389,8 +384,7 @@ MESSAGE
           captured = (value.is_a?(String) ? value : nil)
         end
 
-        return nil if captured.nil?
-        return (haml_buffer.options[:ugly] ? captured : prettify(captured))
+        captured
       end
     ensure
       haml_buffer.capture_position = nil
@@ -417,7 +411,7 @@ MESSAGE
     # @param newline [Boolean] Whether to add a newline after the text
     # @param indent [Boolean] Whether to add indentation to the first line
     def haml_internal_concat(text = "", newline = true, indent = true)
-      if haml_buffer.options[:ugly] || haml_buffer.tabulation == 0
+      if haml_buffer.tabulation == 0
         haml_buffer.buffer << "#{text}#{"\n" if newline}"
       else
         haml_buffer.buffer << %[#{haml_indent if indent}#{text.to_s.gsub("\n", "\n#{haml_indent}")}#{"\n" if newline}]
@@ -501,7 +495,7 @@ MESSAGE
       attrs.keys.each {|key| attrs[key.to_s] = attrs.delete(key)} unless attrs.empty?
       name, attrs = merge_name_and_attributes(name.to_s, attrs)
 
-      attributes = ::Hamlit::HamlCompiler.build_attributes(haml_buffer.html?,
+      attributes = Hamlit::HamlAttributeBuilder.build_attributes(haml_buffer.html?,
         haml_buffer.options[:attr_wrapper],
         haml_buffer.options[:escape_attrs],
         haml_buffer.options[:hyphenate_data_attrs],
@@ -513,8 +507,8 @@ MESSAGE
       end
 
       if flags.include?(:/)
-        raise ::Hamlit::HamlError.new(::Hamlit::HamlError.message(:self_closing_content)) if text
-        raise ::Hamlit::HamlError.new(::Hamlit::HamlError.message(:illegal_nesting_self_closing)) if block
+        raise HamlError.new(HamlError.message(:self_closing_content)) if text
+        raise HamlError.new(HamlError.message(:illegal_nesting_self_closing)) if block
       end
 
       tag = "<#{name}#{attributes}>"
@@ -536,7 +530,7 @@ MESSAGE
       end
 
       if text
-        raise ::Hamlit::HamlError.new(::Hamlit::HamlError.message(:illegal_nesting_line, name))
+        raise HamlError.new(HamlError.message(:illegal_nesting_line, name))
       end
 
       if flags.include?(:<)
@@ -599,9 +593,9 @@ MESSAGE
     end
 
     # Characters that need to be escaped to HTML entities from user input
-    HTML_ESCAPE = { '&' => '&amp;', '<' => '&lt;', '>' => '&gt;', '"' => '&quot;', "'" => '&#039;' }
+    HTML_ESCAPE = {'&' => '&amp;', '<' => '&lt;', '>' => '&gt;', '"' => '&quot;', "'" => '&#39;'}.freeze
 
-    HTML_ESCAPE_REGEX = /[\"><&]/
+    HTML_ESCAPE_REGEX = /['"><&]/
 
     # Returns a copy of `text` with ampersands, angle brackets and quotes
     # escaped into HTML entities.
@@ -613,11 +607,13 @@ MESSAGE
     # @param text [String] The string to sanitize
     # @return [String] The sanitized string
     def html_escape(text)
-      text = text.to_s
-      text.gsub(HTML_ESCAPE_REGEX, HTML_ESCAPE)
+      CGI.escapeHTML(text.to_s)
     end
 
-    HTML_ESCAPE_ONCE_REGEX = /[\"><]|&(?!(?:[a-zA-Z]+|#(?:\d+|[xX][0-9a-fA-F]+));)/
+    # Always escape text regardless of html_safe?
+    alias_method :html_escape_without_haml_xss, :html_escape
+
+    HTML_ESCAPE_ONCE_REGEX = /['"><]|&(?!(?:[a-zA-Z]+|#(?:\d+|[xX][0-9a-fA-F]+));)/
 
     # Escapes HTML entities in `text`, but without escaping an ampersand
     # that is already part of an escaped entity.
@@ -629,9 +625,12 @@ MESSAGE
       text.gsub(HTML_ESCAPE_ONCE_REGEX, HTML_ESCAPE)
     end
 
+    # Always escape text once regardless of html_safe?
+    alias_method :escape_once_without_haml_xss, :escape_once
+
     # Returns whether or not the current template is a Haml template.
     #
-    # This function, unlike other {Haml::Helpers} functions,
+    # This function, unlike other {Hamlit::HamlHelpers} functions,
     # also works in other `ActionView` templates,
     # where it will always return false.
     #
@@ -656,13 +655,13 @@ MESSAGE
       # skip merging if no ids or classes found in name
       return name, attributes_hash unless name =~ /^(.+?)?([\.#].*)$/
 
-      return $1 || "div", ::Hamlit::HamlBuffer.merge_attrs(
-        ::Hamlit::HamlParser.parse_class_and_id($2), attributes_hash)
+      return $1 || "div", HamlAttributeBuilder.merge_attributes!(
+        Hamlit::HamlParser.parse_class_and_id($2), attributes_hash)
     end
 
     # Runs a block of code with the given buffer as the currently active buffer.
     #
-    # @param buffer [Haml::Buffer] The Haml buffer to use temporarily
+    # @param buffer [Hamlit::HamlBuffer] The Haml buffer to use temporarily
     # @yield A block in which the given buffer should be used
     def with_haml_buffer(buffer)
       @haml_buffer, old_buffer = buffer, @haml_buffer
@@ -675,9 +674,9 @@ MESSAGE
       @haml_buffer = old_buffer
     end
 
-    # The current {Haml::Buffer} object.
+    # The current {Hamlit::HamlBuffer} object.
     #
-    # @return [Haml::Buffer]
+    # @return [Hamlit::HamlBuffer]
     def haml_buffer
       @haml_buffer if defined? @haml_buffer
     end
@@ -693,22 +692,6 @@ MESSAGE
       _erbout = _erbout = _hamlout.buffer
       proc { |*args| proc.call(*args) }
     end
-
-    def prettify(text)
-      text = text.split(/^/)
-      text.delete('')
-
-      min_tabs = nil
-      text.each do |line|
-        tabs = line.index(/[^ ]/) || line.length
-        min_tabs ||= tabs
-        min_tabs = min_tabs > tabs ? tabs : min_tabs
-      end
-
-      text.each_with_object('') do |line, str|
-        str << line.slice(min_tabs, line.length)
-      end
-    end
   end
 end
 
@@ -720,8 +703,7 @@ class Object
   # is a proper Haml context.
   # Because `ActionView` helpers may be included in non-`ActionView::Base` classes,
   # it's a good idea to define \{#is\_haml?} for all objects.
-  def is_haml?
-    false
-  end
-  alias :is_haml? :is_haml?
+  # def is_haml?
+  #   false
+  # end
 end
