@@ -13,7 +13,7 @@ describe Hamlit::Engine do
       it { assert_haml(%q|#b{ id: 'a' }|) }
       it { assert_haml(%q|%a{ 'id' => 60 }|) }
       if Gem::Version.new(RUBY_VERSION) >= Gem::Version.new('2.4.0')
-        it { assert_haml(%q|%p{class: "a #{"1".concat "2", "3"}"} foo|) }
+        it { assert_haml(%q|%p{class: "a #{["1", "2", "3"].join}"} foo|) }
       end
 
       it { assert_haml(%q|#a{ id: 'b' }(id=id)|,       locals: { id: 'c' }) }
@@ -25,6 +25,17 @@ describe Hamlit::Engine do
       it { assert_haml(%q|#b{ hash }|,         locals: { hash: { id: 'a' } }) }
       it { assert_haml(%q|#b{ hash }(id='c')|, locals: { hash: { id: 'a' }, id: 'c' }) }
       it { assert_haml(%q|#b{ hash }(id=id)|,  locals: { hash: { id: 'a' }, id: 'c' }) }
+
+      it { assert_haml(<<-HAML.unindent) }
+        .haml#info{
+          "data": {
+            "content": "/:|}",
+            "haml-info": {
+              "url": "https://haml.info",
+            }
+          }
+        } Haml
+      HAML
     end
 
     describe 'incompatibility' do
@@ -264,7 +275,7 @@ describe Hamlit::Engine do
     it { assert_render(%Q|<div class='static' id='static'></div>\n|, %q|.static#static[nil]|) }
     it do
       assert_render(
-        %Q|<a class='dynamic pre_test_object static' id='static_dynamic_pre_test_object_10'></a>\n|,
+        %Q|<a class='static dynamic pre_test_object' id='static_dynamic_pre_test_object_10'></a>\n|,
         %q|%a.static#static[foo, 'pre']{ id: dynamic, class: dynamic }|,
         locals: { foo: TestObject.new(10), dynamic: 'dynamic' },
       )
