@@ -58,5 +58,5 @@ describe Hamlit::Filters do
             alert("#{'<&>'}")
       HAML
     end
-  end
+  end unless /java/ === RUBY_PLATFORM # execjs is not working with Travis JRuby environment
 end
